// Function to read contents of navigation.json
function readTextFile(file, callback) {
    var rawFile = new XMLHttpRequest();
    rawFile.overrideMimeType("application/json");
    rawFile.open("GET", file, true);
    rawFile.onreadystatechange = function() {
        if (rawFile.readyState === 4) {
            callback(rawFile.responseText);
        }
    }
    rawFile.send();
}

var toc = document.createElement("nav");
var main = document.querySelector("main");
var sections = [];
var anchors = [].slice.call(document.querySelectorAll("a"));
var pageLocation = window.location.href.split("/");

var setup = function() {
	// Set the toc session var to the default value (hidden) if it's not already set
	if(window.sessionStorage.getItem("toc") === null) {
		window.sessionStorage.setItem("toc", false);
		// Do nothing else
	}
	// Show the toc if the toc session variable is on
	else if(window.sessionStorage.getItem("toc") == "true") {
		setInitialTocState();
	}

	// Remove any existing selection. It shoudl only be one, but let's check for all just in case
	var selected = toc.querySelectorAll(".selected");
	for(var i = 0; i < selected.length; i++)
		selected[i].classList.remove("selected");

	// Mark the current page as selected
	var currentPageInToc = toc.querySelector("a[href*=\"" + pageLocation[pageLocation.length - 1] + "\"]");
	if(currentPageInToc)
		currentPageInToc.classList.add("selected");

	// Record the width of the window so we can revert to it when closing the toc
	if(window.sessionStorage.getItem("windowWidth") === null ) {
		window.sessionStorage.setItem("windowWidth", document.windowWidth);
	}
	// Add eventlistsners to any showTOCLinks
	var showTocLinks = document.querySelectorAll(".showTocLink");
	for(var i = 0; i < showTocLinks.length; i++) {
		showTocLinks[i].addEventListener("click", toggleToc, false);
	};

	// Reset transitions on tocSections so they can animate
	for(var i = 0; i < sections.length; i++) {
		sections[i].parentNode.addEventListener("click", toggleTocSection, false);
	}

	for(var i = 0; i < anchors.length; i++) {
		anchors[i].addEventListener("click", recordState, false);
	}

	// Check we're in a modern helpviewer before trying to enable the table of contents button in the toolbar
	if("HelpViewer" in window && "showTOCButton" in window.HelpViewer) {
		setTimeout(function() { 
			window.HelpViewer.showTOCButton(true, false, toggleToc);
		}, 1);
	}
}

// Toggls the table of contens when a button or link is clicked
var toggleToc = function() {
	var tocIsVisible = window.sessionStorage.getItem("toc") == "true";
	var showTocLinks = document.querySelectorAll(".showTocLink");
	for(var i = 0; i < showTocLinks.length; i++) {
		showTocLinks[i].innerHTML = (tocIsVisible ? "Show" : "Hide") + " topics";
	};
	main.classList.toggle("toc-on", !tocIsVisible);
	if("HelpViewer" in window && "resizeTo" in window.HelpViewer) {
		if(!tocIsVisible && document.width < 560) {
			window.sessionStorage.setItem("windowWidth", document.width);
			window.HelpViewer.resizeTo(560, 0, .3, "easeInEaseOut");
		}
		else if(tocIsVisible && document.width <= 560)
			window.HelpViewer.resizeTo(parseInt(window.sessionStorage.getItem("windowWidth")), 0, .3, "easeInEaseOut");
	}
	window.sessionStorage.setItem("toc", !tocIsVisible);
	recordState();
};

// Shows the table of contents without setting the session variable
var setInitialTocState = function() {
	main.classList.toggle("toc-on", true);
	main.style.transitionDuration = main.style.WebkitTransitionDuration = "0";
	var showTocLinks = document.querySelectorAll(".showTocLink");
	for(var i = 0; i < showTocLinks.length; i++) {
		showTocLinks[i].innerHTML = "Hide topics";
	};
	setTimeout(function() { main.style.transitionDuration = main.style.WebkitTransitionDuration = "" }, 10);
}

var toggleTocSection = function(e) {
	// Don't run for regular links
	if(!e.target.classList.contains("tocSectionLink")) {
		return false;
	}
	e.stopPropagation();
	this.classList.toggle("expanded");
	var ul = this.querySelector("ul");
	var itemsToResize = [ul];
	var parent = this.parentNode;
	var totalHeight = 0;

	while(!parent.classList.contains("toc")) {
		if(parent.nodeName.toLowerCase() === "ul")
			itemsToResize.push(parent);
		parent = parent.parentNode;
	}
	ul.classList.toggle("hidden");
	if(this.classList.contains("expanded")) {
		for(var i = 0; i < itemsToResize.length; i++)
		{
			totalHeight += itemsToResize[i].scrollHeight;
			itemsToResize[i].style.maxHeight = totalHeight + "px";
		}
	}
	else {
		for(var i = 0; i < itemsToResize.length; i++) {
			if(itemsToResize[i] === ul)
				itemsToResize[i].style.maxHeight = "";
			else
				itemsToResize[i].style.maxHeight = (parseInt(itemsToResize[i].style.maxHeight) - totalHeight) + "px";
			totalHeight += itemsToResize[i].scrollHeight;
		}
	}
	recordState();
};

var setInitialSectionsState = function() {
	// Show sections based on sessionStorage variables
	for(var i = 0; i < sections.length; i++) {
		var sectionTrigger = sections[i].parentNode;
		sectionTrigger.style.transitionDuration = sectionTrigger.style.WebkitTransitionDuration = "0";
		if(sectionTrigger.classList.contains("closed")) {
			sectionTrigger.style.maxHeight = sectionTrigger.firstElementChild.clientHeight + "px";
		}
		// Add event handlers to links
		sectionTrigger.addEventListener("click", toggleTocSection, false);
	}
};

var recordState = function(e) {
	window.sessionStorage.setItem("tocHtml", toc.innerHTML);
};

if(window.sessionStorage.getItem("tocHtml") !== null && window.sessionStorage.getItem("tocHtml") !== "undefined") {
	toc.classList.add("tocContainer");
	toc.innerHTML = window.sessionStorage.getItem("tocHtml");
	document.body.insertBefore(toc, main);
	sections = document.querySelectorAll(".tocSection");

	setup();
}
else {
	// Check for existence of navigation.json before running any toc code
	readTextFile("navigation.json", function(text){
		// Get the contents of navigation.json if it exists 
		// (if it doesn't, an error will be thrown blocking the rest of the script)
	    var tocData = JSON.parse(text);

	    // Build up the table of contents
	    var list = document.createElement("ul");
	    sections = [];

	    toc.classList.add("tocContainer");
	    toc.appendChild(list);

	    for(var tocItem in tocData.toc) {
	    	var listItem = document.createElement("li");
	    	var anchor = document.createElement("a");

	    	listItem.appendChild(anchor);
	    	list.appendChild(listItem);
	    	list.classList.add("toc");
	    	anchor.href = "#";
	    	anchor.classList.add("tocSectionLink");
	    	anchor.innerHTML = tocData.sections[tocData.toc[tocItem]].name;
	    	anchors.push(anchor);

	    	var addChildren = function(parent, children) {
	    		for(var i = 0; i < children.length; i++) {
	    			if(childList === undefined) {
	    				var childList = document.createElement("ul");
	    				parent.appendChild(childList);
	    			}
	    			var childItem = document.createElement("li");
	    			var childAnchor = document.createElement("a");

	    			childList.appendChild(childItem);
	    			childList.classList.add("tocSection", "hidden");
	    			childItem.appendChild(childAnchor);
	    			if(tocData.sections[children[i]]) {
	    				addChildren(childItem, tocData.sections[children[i]].children);
	    				childAnchor.innerHTML = tocData.sections[children[i]].name;
	    				childAnchor.href = "#";
	    				childAnchor.classList.add("tocSectionLink");
	    			}
	    			else {
	    				childAnchor.innerHTML = tocData.topics[children[i]].name;
	    				childAnchor.href = tocData.topics[children[i]].href;
	    			}
	    			anchors.push(childAnchor);
	    		}
	    		sections.push(childList);
	    	}

	    	addChildren(listItem, tocData.sections[tocData.toc[tocItem]].children);
	    }

	    document.body.insertBefore(toc, main);

	    setup();
	});
}